==> nui_form-0.0.52.tar.gz: lib/component/nuif_dropdown.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_form/model/nuif_models.dart';
import 'package:nui_form/nuif_form.dart';
import 'package:nui_form/nuif_form_component.dart';

typedef OnItemSelected = void Function(NUIFormSelection value); //Clicked on next/done

class NUIFDropdownItem extends NUIFormComponent with NUIFValidatable{
  NUIFDropdownItem({
    @required String inpId,
    @required this.selections,
    double width,
    double height,
    double weight,
    bool enabled: true,
    bool compulsory: true,
    bool show: true,
    this.selectedItem,
    this.autofocus: false,
    this.focusNode,
    this.nextFocusNode,
    this.onItemSelected,
    this.style,
    this.hint,
    this.label,
    this.helperText,
    this.icon,
    this.prefix,
    this.prefixText,
    this.prefixIcon,
    this.suffix,
    this.suffixText,
    this.suffixIcon,
    this.showArrow: true,
    Alignment align,
    EdgeInsets margin,
  }) : super(inpId: inpId, inpType:ComponentType.DROPDOWN, compulsory: compulsory, show: show, width: width, height : height, weight: weight, enabled: enabled, margin: margin, align: align);

  String text;
  NUIFSelection selectedItem;
  List<NUIFSelection> selections;
  bool autofocus;
  FocusNode focusNode;
  FocusNode nextFocusNode;
  OnItemSelected onItemSelected;
  String hint;
  String label;
  String helperText;
  Widget icon;
  Widget prefix;
  String prefixText;
  Widget prefixIcon;
  Widget suffix;
  String suffixText;
  Widget suffixIcon;
  NUIFTextStyle style;
  String errorHelperText;
  bool showArrow;

  void onItemSelectedListener(AllItemSelectListener listener, NUIFSelection selection){
      selectedItem = selection;
      NUIFormValue value = NUIFormValue(inpId: selection.getId());
      value.value = NUIFormSelection(value: selectedItem?.getCaption(), code: selectedItem?.getValue());
      this.formValue = value;
      listener(inpId, value.value, true);
  }

  @override
  Widget toWidget({AllButtonClickListener allButtonClickListener, AllTextChangeListener allTextChangeListener, AllItemSelectListener allItemSelectListener, Key key}) {
    return NUIFDropdown(item: this, allItemSelectListener: allItemSelectListener, key: key);
  }

  @override
  bool validate(bool validate){
    final curValue = formValue?.value?.value;
    logNUI("NUIFDropDown", "Validating dropdown of id : $inpId, current value : $curValue");

    bool validatedAll = false;
    if(compulsory == true && curValue.isNullOrEmpty()){
      failed = true;
      formValue?.validated = !failed;
      errorHelperText = errorMessages[TextValidation.NOT_FILLED];
      validatedAll = false;
    }
    else {
      var validateSuccess = true;
      failed = !validateSuccess;
      validated = validateSuccess;

      validatedAll = validateSuccess;
    }

    if(stateGetter != null){
      stateGetter().update(this);
    }
    return validatedAll;
  }

  @override
  void extractValue(Map<String, NUIFormValue> map){
    map[inpId] = formValue;
  }

  OutlineInputBorder getBorder(){
    return style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getDisabledBorder(){
    return style?.disabledBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getEnabledBorder(){
    return style?.enabledBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getFocusedBorder(){
    return style?.focusedBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getErrorBorder(){
    return style?.errorBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getErrorFocusedBorder(){
    return style?.errorFocusedBorder ?? style?.border ?? OutlineInputBorder();
  }
}

class NUIFDropdown extends StatefulWidget {
  NUIFDropdown({@required this.item, this.allItemSelectListener, Key key}): super(key: key);
  final NUIFDropdownItem item;
  final AllItemSelectListener allItemSelectListener;
  @override
  _NUIFDropdownState createState() => _NUIFDropdownState();
}

class _NUIFDropdownState extends NUIFState<NUIFDropdown, NUIFDropdownItem> {
  TextEditingController textController;

  @override
  void initState(){
    this.item = widget.item;
    super.initState();
    textController = TextEditingController(text: item.selectedItem?.getCaption() ?? "");
  }

  Stack _bindWithDropdownItem(NUIFDropdownItem item){
    InputDecoration decoration;
    if(item.style?.design == NUIFTextDesign.NORMAL || item.style?.design == NUIFTextDesign.BORDER){
      Widget suffixIcon = item.suffixIcon ?? Icon(Icons.arrow_drop_down, size: 24, color: item.style.labelStyle.color);

      decoration = InputDecoration(
        hintText: item.hint,
        hintStyle: item.style?.hintStyle,
        labelText: item.label,
        labelStyle: item.style?.labelStyle,
        icon: item.icon,
        prefix: item.prefix,
        prefixText: item.prefixText,
        prefixIcon: item.prefixIcon,
        suffix: item.suffix,
        suffixText: item.suffixText,
        suffixIcon: item.showArrow != false ? suffixIcon : null,
        helperText: item.helperText,
        helperStyle: item.style?.helperStyle,
        isDense: item.style?.dense,
        contentPadding: item.style?.contentPadding,
        border: item.style?.design == NUIFTextDesign.BORDER ? item.getBorder() : null,
        enabledBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getEnabledBorder() : null,
        disabledBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getDisabledBorder() : null,
        focusedBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getFocusedBorder() : null,
        errorBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getErrorBorder() : null,
        focusedErrorBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getErrorFocusedBorder() : null,
      );
    }
    else{
      decoration = InputDecoration.collapsed(
        hintText: item.hint,
        hintStyle: item.style?.hintStyle,
      );
    }

    if(item.formValue != null) {
      for (NUIFSelection sel in item.selections) {
        if (match(sel.getId(), item.formValue.inpId)){
          item.selectedItem = sel;
          break;
        }
      }
    }

    textController.text = item.selectedItem?.getCaption();

    final textfield = TextField(
      controller: textController,
      autofocus: item.autofocus,
      enabled: false,
      focusNode: item.focusNode,
      style: item.style?.textStyle,
      cursorColor: item.style?.cursor?.color ?? null,
      cursorRadius: item.style?.cursor?.cursorRadius ?? null,
      cursorWidth: item.style?.cursor?.cursorWidth ?? 2,
      decoration: decoration,
    );

    final selections = item.selections.map((e){
      return DropdownMenuItem(
          child: e.getCustomView() ?? Text(e.getCaption(), style: e.getFontStyle() ?? TextStyle(color: NUIColors.NUITextBlack, fontSize: 14)),
          value: e,
      );
    }).toList();

    final dropdown = DropdownButtonHideUnderline(
        child: DropdownButton(
          iconEnabledColor: NUIColors.NUITransparent,
          iconDisabledColor: NUIColors.NUITransparent,
          elevation: 2,
          iconSize: 0,
          isExpanded: true,
          items: selections,
          onChanged: (value){
            if(value is NUIFSelection){
              setState(() {
                item.onItemSelectedListener(widget.allItemSelectListener, value);
              });
            }
          },
      )
    );

    final stack = Stack(
      alignment: Alignment.centerLeft,
      children: [
        textfield,
        Padding(
          padding: const EdgeInsets.only(left: 15, right: 20),
          child: dropdown,
        )
      ],
    );

    return stack;
  }

  @override
  void dispose(){
    super.dispose();
    textController.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return item.wrapWidget(_bindWithDropdownItem(item));
  }
}

==> nui_form-0.0.56.tar.gz: lib/component/nuif_dropdown.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_form/model/nuif_models.dart';
import 'package:nui_form/nuif_form.dart';
import 'package:nui_form/nuif_form_component.dart';

typedef OnItemSelected = void Function(NUIFormSelection value); //Clicked on next/done

class NUIFDropdownItem extends NUIFormComponent with NUIFValidatable{
  NUIFDropdownItem({
    @required String inpId,
    @required this.selections,
    double width,
    double height,
    double weight,
    bool enabled: true,
    bool compulsory: true,
    bool show: true,
    this.selectedItem,
    this.autofocus: false,
    this.focusNode,
    this.nextFocusNode,
    this.onItemSelected,
    this.style,
    this.hint,
    this.label,
    this.helperText,
    this.icon,
    this.prefix,
    this.prefixText,
    this.prefixIcon,
    this.suffix,
    this.suffixText,
    this.suffixIcon,
    this.showArrow: true,
    Alignment align,
    EdgeInsets margin,
  }) : super(inpId: inpId, inpType:ComponentType.DROPDOWN, compulsory: compulsory, show: show, width: width, height : height, weight: weight, enabled: enabled, margin: margin, align: align);

  String text;
  NUIFSelection selectedItem;
  List<NUIFSelection> selections;
  bool autofocus;
  FocusNode focusNode;
  FocusNode nextFocusNode;
  OnItemSelected onItemSelected;
  String hint;
  String label;
  String helperText;
  Widget icon;
  Widget prefix;
  String prefixText;
  Widget prefixIcon;
  Widget suffix;
  String suffixText;
  Widget suffixIcon;
  NUIFTextStyle style;
  String errorHelperText;
  bool showArrow;

  void onItemSelectedListener(AllItemSelectListener listener, NUIFSelection selection){
      selectedItem = selection;
      NUIFormValue value = NUIFormValue(inpId: selection.getId());
      value.value = NUIFormSelection(value: selectedItem?.getCaption(), code: selectedItem?.getValue());
      this.formValue = value;
      if(listener != null) listener(inpId, value.value, true);
  }

  @override
  Widget toWidget({AllButtonClickListener allButtonClickListener, AllTextChangeListener allTextChangeListener, AllItemSelectListener allItemSelectListener, Key key}) {
    return NUIFDropdown(item: this, allItemSelectListener: allItemSelectListener, key: key);
  }

  @override
  bool validate(bool validate){
    final curValue = formValue?.value?.value;
    logNUI("NUIFDropDown", "Validating dropdown of id : $inpId, current value : $curValue");

    bool validatedAll = false;
    if(compulsory == true && curValue.isNullOrEmpty()){
      failed = true;
      formValue?.validated = !failed;
      errorHelperText = errorMessages[TextValidation.NOT_FILLED];
      validatedAll = false;
    }
    else {
      var validateSuccess = true;
      failed = !validateSuccess;
      validated = validateSuccess;

      validatedAll = validateSuccess;
    }

    if(stateGetter != null){
      stateGetter().update(this);
    }
    return validatedAll;
  }

  @override
  void extractValue(Map<String, NUIFormValue> map){
    map[inpId] = formValue;
  }

  OutlineInputBorder getBorder(){
    return style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getDisabledBorder(){
    return style?.disabledBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getEnabledBorder(){
    return style?.enabledBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getFocusedBorder(){
    return style?.focusedBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getErrorBorder(){
    return style?.errorBorder ?? style?.border ?? OutlineInputBorder();
  }
  OutlineInputBorder getErrorFocusedBorder(){
    return style?.errorFocusedBorder ?? style?.border ?? OutlineInputBorder();
  }

  @override
  void fillInValue(Map<String, NUIFormValue> values) {
    final formValue = values[inpId];
    if(formValue == null) return;

    for(NUIFSelection sel in selections){
      if(match(formValue.inpId, sel.getId()) || match(formValue.value?.value, sel.getValue())){
        if(stateGetter != null){
          onItemSelectedListener(null, sel);
          stateGetter().update(this);
        }
        break;
      }
    }
  }
}

class NUIFDropdown extends StatefulWidget {
  NUIFDropdown({@required this.item, this.allItemSelectListener, Key key}): super(key: key);
  final NUIFDropdownItem item;
  final AllItemSelectListener allItemSelectListener;
  @override
  _NUIFDropdownState createState() => _NUIFDropdownState();
}

class _NUIFDropdownState extends NUIFState<NUIFDropdown, NUIFDropdownItem> {
  TextEditingController textController;

  @override
  void initState(){
    this.item = widget.item;
    super.initState();
    textController = TextEditingController(text: item.selectedItem?.getCaption() ?? "");
  }

  Stack _bindWithDropdownItem(NUIFDropdownItem item){
    InputDecoration decoration;
    if(item.style?.design == NUIFTextDesign.NORMAL || item.style?.design == NUIFTextDesign.BORDER){
      Widget suffixIcon = item.suffixIcon ?? Icon(Icons.arrow_drop_down, size: 24, color: item.style.labelStyle.color);

      decoration = InputDecoration(
        hintText: item.hint,
        hintStyle: item.style?.hintStyle,
        labelText: item.label,
        labelStyle: item.style?.labelStyle,
        icon: item.icon,
        prefix: item.prefix,
        prefixText: item.prefixText,
        prefixIcon: item.prefixIcon,
        suffix: item.suffix,
        suffixText: item.suffixText,
        suffixIcon: item.showArrow != false ? suffixIcon : null,
        helperText: item.helperText,
        helperStyle: item.style?.helperStyle,
        isDense: item.style?.dense,
        contentPadding: item.style?.contentPadding,
        border: item.style?.design == NUIFTextDesign.BORDER ? item.getBorder() : null,
        enabledBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getEnabledBorder() : null,
        disabledBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getDisabledBorder() : null,
        focusedBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getFocusedBorder() : null,
        errorBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getErrorBorder() : null,
        focusedErrorBorder: item.style?.design == NUIFTextDesign.BORDER ? item.getErrorFocusedBorder() : null,
      );
    }
    else{
      decoration = InputDecoration.collapsed(
        hintText: item.hint,
        hintStyle: item.style?.hintStyle,
      );
    }

    if(item.formValue != null) {
      for (NUIFSelection sel in item.selections) {
        if (match(sel.getId(), item.formValue.inpId)){
          item.selectedItem = sel;
          break;
        }
      }
    }

    textController.text = item.selectedItem?.getCaption();

    final textfield = TextField(
      controller: textController,
      autofocus: item.autofocus,
      enabled: false,
      focusNode: item.focusNode,
      style: item.style?.textStyle,
      cursorColor: item.style?.cursor?.color ?? null,
      cursorRadius: item.style?.cursor?.cursorRadius ?? null,
      cursorWidth: item.style?.cursor?.cursorWidth ?? 2,
      decoration: decoration,
    );

    final selections = item.selections.map((e){
      return DropdownMenuItem(
          child: e.getCustomView() ?? Text(e.getCaption(), style: e.getFontStyle() ?? TextStyle(color: NUIColors.NUITextBlack, fontSize: 14)),
          value: e,
      );
    }).toList();

    final dropdown = DropdownButtonHideUnderline(
        child: DropdownButton(
          iconEnabledColor: NUIColors.NUITransparent,
          iconDisabledColor: NUIColors.NUITransparent,
          elevation: 2,
          iconSize: 0,
          isExpanded: true,
          items: selections,
          onChanged: (value){
            if(value is NUIFSelection){
              setState(() {
                item.onItemSelectedListener(widget.allItemSelectListener, value);
              });
            }
          },
      )
    );

    final stack = Stack(
      alignment: Alignment.centerLeft,
      children: [
        textfield,
        Padding(
          padding: const EdgeInsets.only(left: 15, right: 20),
          child: dropdown,
        )
      ],
    );

    return stack;
  }

  @override
  void dispose(){
    super.dispose();
    textController.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return item.wrapWidget(_bindWithDropdownItem(item));
  }
}
